<?php

use Symfony\Component\Console\Output\ConsoleOutput;
use Drupal\simple_survey\Entity\SimpleSurvey;

/**
 * Implements hook_drush_command().
 */
function simple_survey_drush_command() {
  $commands = [];

  if (class_exists('Google\Cloud\BigQuery\BigQueryClient')) {
    $commands['send-responses-to-bigquery'] = [
      'description' => 'Send survey tool responses to Big Query',
      'drupal dependencies' => ['simple_survey'],
      'aliases' => ['ss:srbq']
    ];

    $commands['export-to-bigquery'] = [
      'description' => 'Send survey tool responses to Big Query',
      'drupal dependencies' => ['simple_survey'],
      'aliases' => ['ss:ec']
    ];
  }

  return $commands;
}

/**
 * Call back function drush_custom_drush_command_send_responses_to_bigquery()
 * The call back function name in the  following format
 *   drush_{module_name}_{item_id_for_command}()
 */
function drush_simple_survey_send_responses_to_bigquery() {
  /**
   * @var \Drupal\simple_survey\Service\ResponseExportService $responseExport
   *   The response export service.
   */
  $responseExport = \Drupal::service('simple_survey.response_export');
  /**
   * @var \Drupal\simple_survey\Service\BigQueryConnect $bigQuery
   *   The bigQueryConnect Service.
   */
  $bigQuery = \Drupal::service('simple_survey.big_query');
  $surveys = SimpleSurvey::loadMultiple();

  $output = new ConsoleOutput();

  foreach ($surveys as $survey) {
    $bigQuery->setSurvey($survey);
    $responseExport->survey = $survey;

    if (!$bigQuery->checkConfigured()) {
      $output->writeln('<error>Big Query is not configured.</error>');
    }
    else {
      $output->writeln('<info>Converting Data for ' . $survey->label() . '</info>');

      $csv = $responseExport->convertDataToCsv();

      $output->writeln('<info>Submitting to BigQuery for ' . $survey->label() . '</info>');

      if ($bigQuery->updateTable($survey->bigQueryDataset, $survey->bigQueryTable, $csv)) {
        $output->writeln('<info>Submission Complete for ' . $survey->label() . '.</info>');
      }
      else {
        $output->writeln('<error>Submission Failed for ' . $survey->label() . '. Check the error log for more details.</error>');
      }
    }
  }

  /**
   * Creates csv files for all or a single survey.
   *
   * @param string $surveyId
   *   Argument provided to the drush command.
   *
   * @command simple_survey:exportToCsv
   * @aliases ss:ec
   * @options arr An option that takes multiple values.
   * @usage export-to-bigquery default
   *   Sends the default survey response to BigQuery.
   */
  function drush_simple_survey_export_to_bigquery($surveyId = NULL) {
    $output = new ConsoleOutput();

    $output->writeln('<info>Loading Survey</info>');

    if ($surveyId) {
      $simpleSurvey = SimpleSurvey::load($surveyId);
      $filename = $simpleSurvey->id() . '-survey-export.csv';
      $batch = [
        'title' => t('Send Data to BigQuery for %survey', [
          '%survey' => $simpleSurvey->label(),
        ]),
        'operations' => [
          [
            '\Drupal\simple_survey\Service\ResponseExportService::processData',
            [
              $filename,
              $surveyId,
            ],
          ],
          [
            '\Drupal\simple_survey\Controller\SimpleSurveyController::submitData',
            [
              $filename,
              $surveyId,
              $simpleSurvey->get('bigQueryDataset'),
              $simpleSurvey->get('bigQueryTable'),
            ],
          ],
        ],
        'progressive' => FALSE,
        'finished' => '\Drupal\simple_survey\Controller\SimpleSurveyController::finishedProcessingBq',
        'init_message' => t('Initializing BiqQuery connection for @source', [
          '@source' => $simpleSurvey->label(),
        ]),
        'progress_message' => t('Data being sent to BiqQuery for @source. Time left: @estimate', [
          '@source' => $simpleSurvey->label(),
        ]),
        'error_message' => t('The export progress has encountered an error.'),
      ];

      batch_set($batch);

      drush_backend_batch_process();

    }
    else {
      $output->writeln('<error>No survey found.</error>');
    }
  }

}
