<?php

namespace Drupal\simple_survey\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Datetime\DateFormatter;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use SplTempFileObject;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Messenger\Messenger;
use Drupal\Core\Url;
use SplFileObject;

/**
 * Survey Response convert and export service.
 */
class ResponseExportService implements ContainerInjectionInterface {

  /**
   * The module name.
   *
   * @var string
   */
  private $moduleName = 'simple_survey';

  /**
   * Drupal config service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  private $configFactory;

  /**
   * Drupal logger service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Drupal messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The Drupal Time Service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * Date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormatter;

  /**
   * The Entity Type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The current survey.
   *
   * @var \Drupal\simple_survey\Entity\SimpleSurvey
   */
  public $survey;

  /**
   * PHP File Object.
   *
   * @var \SplTempFileObject
   */
  public $fileObject;

  /**
   * The export file name.
   *
   * @var string
   */
  public $filename;

  /**
   * Header keys.
   *
   * @var array
   */
  public $headerColumnKeys = [
    'drupal_id',
    'cookie_uuid',
    'site_path',
    'langcode',
  ];

  /**
   * Service config and DI.
   *
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   Drupal Messenger service.
   * @param \Drupal\Core\Datetime\DateFormatter $dateFormatter
   *   Date formatter service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerFactory
   *   Drupal logger service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   Drupal config factory service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity Type Manager service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   Time service.
   */
  public function __construct(
    MessengerInterface $messenger,
    DateFormatter $dateFormatter,
    LoggerChannelFactoryInterface $loggerFactory,
    ConfigFactoryInterface $configFactory,
    EntityTypeManagerInterface $entity_type_manager,
    TimeInterface $time
  ) {
    $this->messenger = $messenger;
    $this->configFactory = $configFactory;
    $this->dateFormatter = $dateFormatter;
    $this->entityTypeManager = $entity_type_manager;
    $this->logger = $loggerFactory->get($this->moduleName);
    $this->time = $time;

    $this->simpleSurveyConfig = $this->configFactory->getEditable($this->moduleName . '.settings');
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('messenger'),
      $container->get('date.formatter'),
      $container->get('logger.factory'),
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('datetime.time')
    );
  }

  /**
   * Builds the csv header row and sets column keys.
   *
   * @return array
   *   Returns an array of header data.
   */
  public static function createHeaderRow(string $survey) {
    $header = [
      'columns' => [],
      'keys' => [
        'drupal_id',
        'cookie_uuid',
        'site_path',
        'langcode',
      ],
    ];

    $header['columns'] = [
      preg_replace('/[^A-Za-z0-9_]/', '', strtolower(str_replace(' ', '_', 'Drupal ID'))),
      preg_replace('/[^A-Za-z0-9_]/', '', strtolower(str_replace(' ', '_', 'User'))),
      preg_replace('/[^A-Za-z0-9_]/', '', strtolower(str_replace(' ', '_', 'Page Submitted On'))),
      preg_replace('/[^A-Za-z0-9_]/', '', strtolower(str_replace(' ', '_', 'Language'))),
    ];

    $query = \Drupal::entityTypeManager()->getStorage('survey_question')->getQuery();

    $query->condition('survey', $survey);

    $nids = $query
      ->execute();

    /** @var Drupal/simple_survey/Entity/SurveyQuestion $questions */
    $questions = \Drupal::entityTypeManager()->getStorage('survey_question')
      ->loadMultiple($nids);

    foreach ($questions as $question) {
      if ($question->get('question_type')->value == 'normal') {
        $sanitizeQuestion = strip_tags($question->get('question')->value);
        $sanitizeQuestion = str_replace('&nbsp;', ' ', $sanitizeQuestion);
        $sanitizeQuestion = str_replace(["\r", "\n"], '', $sanitizeQuestion);
        $header['columns'][] = preg_replace('/[^A-Za-z0-9_]/', '', strtolower(str_replace(' ', '_', $sanitizeQuestion)));
        $header['keys'][] = $question->id();
      }
    }

    $header['columns'][] = preg_replace('/[^A-Za-z0-9_]/', '', strtolower(str_replace(' ', '_', 'Submission was created')));
    $header['keys'][] = 'created';

    return $header;
  }

  /**
   * Builds the csv header row and sets column keys.
   */
  public function outputHeaderRow() {

    $headerColumns = [
      preg_replace('/[^A-Za-z0-9_]/', '', strtolower(str_replace(' ', '_', 'Drupal ID'))),
      preg_replace('/[^A-Za-z0-9_]/', '', strtolower(str_replace(' ', '_', 'User'))),
      preg_replace('/[^A-Za-z0-9_]/', '', strtolower(str_replace(' ', '_', 'Page Submitted On'))),
      preg_replace('/[^A-Za-z0-9_]/', '', strtolower(str_replace(' ', '_', 'Language'))),
    ];

    $query = $this->entityTypeManager->getStorage('survey_question')->getQuery();

    if ($this->survey) {
      $query->condition('survey', $this->survey->id());
    }

    $nids = $query
      ->execute();

    /** @var Drupal/simple_survey/Entity/SurveyQuestion $questions */
    $questions = $this->entityTypeManager->getStorage('survey_question')
      ->loadMultiple($nids);

    foreach ($questions as $question) {
      if ($question->get('question_type')->value == 'normal') {
        $sanitizeQuestion = strip_tags($question->get('question')->value);
        $sanitizeQuestion = str_replace('&nbsp;', ' ', $sanitizeQuestion);
        $sanitizeQuestion = str_replace(["\r", "\n"], '', $sanitizeQuestion);
        $headerColumns[] = preg_replace('/[^A-Za-z0-9_]/', '', strtolower(str_replace(' ', '_', $sanitizeQuestion)));
        $this->headerColumnKeys[] = $question->id();
      }
    }

    $headerColumns[] = preg_replace('/[^A-Za-z0-9_]/', '', strtolower(str_replace(' ', '_', 'Submission was created')));
    $this->headerColumnKeys[] = 'created';
    $this->fileObject->fputcsv($headerColumns);
  }

  /**
   * Gets number of responses for current survey.
   *
   * @return int
   *   The number of responses.
   */
  public static function getResponseCount($survey) {
    $query = \Drupal::entityTypeManager()->getStorage('survey_response')->getQuery();

    if ($survey) {
      $query->condition('survey', $survey);
    }

    return $query
      ->count()
      ->execute();
  }

  /**
   * Builds a response data rray.
   */
  public static function createCsvRow($keys, $id) {
    $outputResponse = [];

    foreach ($keys as $key) {
      $outputResponse[$key] = NULL;
    }

    /** @var Drupal/simple_survey/Entity/SurveyResponse $surveyResponse */
    $surveyResponse = \Drupal::entityTypeManager()->getStorage('survey_response')->load($id);

    $outputResponse['drupal_id'] = $surveyResponse->id();
    $outputResponse['cookie_uuid'] = $surveyResponse->get('cookie_uuid')->value;
    $outputResponse['site_path'] = $surveyResponse->get('site_path')->value;
    $outputResponse['langcode'] = $surveyResponse->get('langcode')->value;
    $outputResponse['created'] = \Drupal::service('date.formatter')->format($surveyResponse->get('created')->value, 'custom', 'Y-m-d H:i:s');

    $query = \Drupal::entityTypeManager()->getStorage('survey_answer')->getQuery();

    $saids = $query
      ->condition('parent_uuid', $surveyResponse->get('uuid')->value)
      ->execute();

    /** @var Drupal/simple_survey/Entity/SurveyAnswer $survey_answers */
    $survey_answers = \Drupal::entityTypeManager()->getStorage('survey_answer')
      ->loadMultiple($saids);

    foreach ($survey_answers as $survey_answer) {
      $outputResponse[$survey_answer->get('question_id')->value] = $survey_answer->get('answer')->value;
    }

    return $outputResponse;
  }

  /**
   * Builds a response data rray.
   */
  private function outputResponseData() {
    $headerColumnKeys = $this->outputHeaderRow();

    $query = $this->entityTypeManager->getStorage('survey_response')->getQuery();

    if ($this->survey) {
      $query->condition('survey', $this->survey->id());
    }

    $srids = $query
      ->sort('created', 'DESC')
      ->execute();

    /** @var Drupal/simple_survey/Entity/SurveyResponse $survey_responses */
    $survey_responses = $this->entityTypeManager->getStorage('survey_response')
      ->loadMultiple($srids);

    foreach ($survey_responses as $survey_response) {
      $outputResponse = [];

      foreach ($headerColumnKeys as $key) {
        $outputResponse[$key] = NULL;
      }

      $outputResponse['drupal_id'] = $survey_response->id();
      $outputResponse['cookie_uuid'] = $survey_response->get('cookie_uuid')->value;
      $outputResponse['site_path'] = $survey_response->get('site_path')->value;
      $outputResponse['langcode'] = $survey_response->get('langcode')->value;
      $outputResponse['created'] = $this->dateFormatter->format($survey_response->get('created')->value, 'custom', 'Y-m-d H:i:s');

      $query = $this->entityTypeManager->getStorage('survey_answer')->getQuery();

      $saids = $query
        ->condition('parent_uuid', $survey_response->get('uuid')->value)
        ->execute();

      /** @var Drupal/simple_survey/Entity/SurveyAnswer $survey_answers */
      $survey_answers = $this->entityTypeManager->getStorage('survey_answer')
        ->loadMultiple($saids);

      foreach ($survey_answers as $survey_answer) {
        $outputResponse[$survey_answer->get('question_id')->value] = $survey_answer->get('answer')->value;
      }

      $this->fileObject->fputcsv($outputResponse);
    }

  }

  /**
   * Converts survey responses to CSV.
   */
  public function convertDataToCsv() {

    $this->fileObject = new SplTempFileObject();

    $this->outputResponseData();
  }

  /**
   * Get the file object size.
   *
   * @return int
   */
  public function getFileSize() {
    return $this->fileObject->getSize();
  }

  public function outputToBrowser() {
    $this->fileObject->rewind();
    $this->fileObject->fpassthru();
  }

  /**
   * Batch process method for survey responses.
   *
   * @param string $filename
   *   The filename.
   * @param string $survey
   *   The survey machine name.
   * @param array|object $context
   *   The batch process context.
   */
  public static function processData(string $filename, string $survey, &$context) {
    $batch_size = 200;
    $filepath = \Drupal::service('file_system')->realpath("temporary://")
      . '/' . $filename;

    if (empty($context['sandbox'])) {
      $file = new SplFileObject($filepath, 'w');
      $context['sandbox']['headers'] = self::createHeaderRow($survey);
      $context['sandbox']['max'] = self::getResponseCount($survey);
      $context['sandbox']['progress'] = 0;
      $context['results']['filepath'] = $filepath;
      $context['results']['filename'] = $filename;
      $context['results']['survey'] = $survey;
      $file->fputcsv($context['sandbox']['headers']['columns']);
    }
    else {
      $file = new SplFileObject($filepath, 'a');
    }

    $query = \Drupal::entityTypeManager()->getStorage('survey_response')->getQuery();

    $query->condition('survey', $survey);

    $max = $context['sandbox']['progress'] + $batch_size;

    $srids = $query
      ->sort('created', 'DESC')
      ->range($context['sandbox']['progress'], $batch_size)
      ->execute();

    if ($max > $context['sandbox']['max']) {
      $max = $context['sandbox']['max'];
    }

    $context['message'] = t('Adding rows @starting through @max of @total to @filename.<br>
    Current Size: @fileSize.', [
      '@starting' => $context['sandbox']['progress'],
      '@max' => $max,
      '@total' => $context['sandbox']['max'],
      '@filename' => $filename,
      '@fileSize' => format_size($file->getSize()),
    ]);

    foreach ($srids as $srid) {
      $outputResponse = self::createCsvRow($context['sandbox']['headers']['keys'], $srid);

      $file->fputcsv($outputResponse);
      $context['sandbox']['progress']++;
    }

    if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
      $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
    }
  }

  /**
   * CSV file creation atch process finished method.
   *
   * @param bool $success
   *   Boolean value that specifies batch success.
   * @param array $results
   *   Array that contains results from batch context.
   * @param array $operations
   *   Array that contains batch operations.
   */
  public static function finishedFileProcessing(bool $success, array $results, array $operations) {
    $status = Messenger::TYPE_STATUS;

    if ($success) {
      $message = t('CSV File created. This file will be available for 6 hours.
      <br>Download: <a href=":link" title=":filename">:filename</a>', [
        ':link' => Url::fromRoute('simple_survey.download_responses', [
          'simple_survey' => $results['survey'],
          'filename' => $results['filename'],
        ])->toString(),
        ':filename' => $results['filename'],
      ]);

      \Drupal::logger('simple_survey')->info($message);
    }
    else {
      $message = t('Failed to build CSV File, :filename.', [
        ':filename' => $results['filename'],
      ]);

      $status = Messenger::TYPE_ERROR;
      \Drupal::logger('simple_survey')->error($message);
    }

    \Drupal::messenger()->addMessage($message, $status);
  }

}
